/*
 * Copyright (c) 2018-2019. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.wiki;

import java.util.LinkedList;
import java.util.List;

import org.apache.commons.configuration.HierarchicalConfiguration;
import org.apache.commons.configuration.XMLConfiguration;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

import org.hedgecode.chess.AbstractPositionTest;
import org.hedgecode.chess.position.Builder;
import org.hedgecode.chess.position.Position;

/**
 * Tests for {@link WikiBuilder}.
 *
 * @author Dmitry Samoshin aka gotty
 */
@RunWith(JUnit4.class)
public class WikiBuilderTest extends AbstractPositionTest implements WikiPositionTest {

    private static final String LINE_REGEX = "\\r?\\n";

    @Test
    public void testBuild() throws Exception {
        Builder wikiBuilder = WikiBuilder.getInstance();
        for (HierarchicalConfiguration testConfig : getXMLConfig().configurationsAt(TESTS)) {
            setPositionName(
                    String.format(
                            MSG_WIKI_POSITION, testConfig.getString(AbstractPositionTest.NAME)
                    )
            );
            String wiki = testConfig.getString(WIKI_TAG);
            Position position = assignPosition(
                    testConfig.configurationAt(AbstractPositionTest.POSITION)
            );
            boolean withTemplate = false;
            if (testConfig.containsKey(WIKI_TEMPLATE)) {
                withTemplate = testConfig.getBoolean(WIKI_TEMPLATE);
            }
            Wiki.setTemplate(withTemplate);
            assertStringEquals(
                    wiki,
                    wikiBuilder.build(position)
            );
        }
    }

    @Override
    protected void assertStringEquals(String expected, String actual) {
        Assert.assertNotNull(expected);
        Assert.assertNotNull(actual);
        String[] expectedLines = expected.split(LINE_REGEX);
        String[] actualLines = actual.split(LINE_REGEX);
        Assert.assertEquals(
                expectedLines.length,
                actualLines.length
        );
        for (int i = 0; i < expectedLines.length; ++i) {
            super.assertStringEquals(
                    expectedLines[i].trim(),
                    actualLines[i].trim()
            );
        }
    }

    @Override
    public List<XMLConfiguration> getXMLPositions() throws Exception {
        List<XMLConfiguration> xmlPositions = new LinkedList<>();
        for (HierarchicalConfiguration testConfig : getXMLConfig().configurationsAt(TESTS)) {
            xmlPositions.add(
                    new XMLConfiguration(
                            testConfig.configurationAt(AbstractPositionTest.POSITION)
                    )
            );
        }
        return xmlPositions;
    }

}