/*
 * Copyright (c) 2018. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.uci.command.from;

import org.hedgecode.chess.uci.Acceptor;
import org.hedgecode.chess.uci.Engine;
import org.hedgecode.chess.uci.UCICommand;
import org.hedgecode.chess.uci.UCIConstants;
import org.hedgecode.chess.uci.annotation.Command;
import org.hedgecode.chess.uci.annotation.CommandDirection;
import org.hedgecode.chess.uci.command.AbstractCommand;
import org.hedgecode.chess.uci.command.OptionParams;

/**
 *
 *
 * @author Dmitry Samoshin aka gotty
 */
@Command(
        name = UCIConstants.OPTION,
        direction = CommandDirection.FROM_ENGINE
)
public class OptionCommand extends AbstractCommand implements UCICommand {

    @Override
    public void exec(final Engine engine, final Acceptor acceptor, String params) {
        OptionParams optionParams = OptionParams.Factory.parse(params);
        if (optionParams != null) {
            switch (optionParams.getType()) {
                case CHECK:
                    acceptor.setOption(
                            optionParams.getName(),
                            Boolean.parseBoolean(optionParams.getDefValue())
                    );
                    break;
                case SPIN:
                    acceptor.setOption(
                            optionParams.getName(),
                            Integer.parseInt(optionParams.getDefValue()),
                            optionParams.getMinValue(),
                            optionParams.getMaxValue()
                    );
                    break;
                case COMBO:
                    acceptor.setOption(
                            optionParams.getName(),
                            optionParams.getDefValue(),
                            optionParams.getVars()
                    );
                    break;
                case BUTTON:
                    acceptor.setOption(
                            optionParams.getName()
                    );
                    break;
                case STRING:
                    acceptor.setOption(
                            optionParams.getName(),
                            optionParams.getDefValue()
                    );
                    break;
            }
        }
    }

}
